--TEST--
Integration test for a custom cat scoreboard.
--FILE--
<?php

require_once 'vendor/autoload.php';

use NyanCat\Cat;
use NyanCat\Rainbow;
use NyanCat\Team;
use NyanCat\Scoreboard;

use Fab\SuperFab;

$scoreboard = new Scoreboard(
    new Cat(
        array(
            array(
                '              ___  ',
                '             / %_) ',
                '     _.----./ /    ',
                '    /        /     ',
                ' __/ (  | (  |     ',
                "/__.-'|_|--|_|     "
            ),
            array(
                '               ___ ',
                '              / %_)',
                '     _.----._/ /   ',
                '    /         /    ',
                ' __/ (  | (  |     ',
                "/__.-|_|--|_|      "
            ),
            array(
                '              ___  ',
                '             / %_) ',
                '    _.----._/ /    ',
                '   /         /     ',
                ' _/ (  / /  /      ',
                "/_/-|_/--|_/       "
            )
        )
    ),
    new Rainbow(
        new SuperFab()
    ),
    array(
        new Team('pass', 'green', '^'),
        new Team('fail', 'red', 'o'),
        new Team('pending', 'cyan', '-'),
    )
);

$scoreboard->start();
for($i = 0; $i <= 2; $i++) {
    $scoreboard->score('pass');
    $scoreboard->score('fail');
    $scoreboard->score('pending');
}
$scoreboard->stop();

--EXPECT--
 
 [32m0[0m
 [31m0[0m
 [36m0[0m
 
 
[6A[5C 
[5C[38;5;154m-[0m
[5C[38;5;154m-[0m
[5C[38;5;154m-[0m
[5C[38;5;154m-[0m
[5C 
[6A[6C              ___  
[6C             / -_) 
[6C     _.----./ /    
[6C    /        /     
[6C __/ (  | (  |     
[6C/__.-'|_|--|_|     
[6A 
 [32m1[0m
 [31m0[0m
 [36m0[0m
 
 
[6A[5C  
[5C[38;5;154m-[0m[38;5;154m_[0m
[5C[38;5;154m-[0m[38;5;154m_[0m
[5C[38;5;154m-[0m[38;5;154m_[0m
[5C[38;5;154m-[0m[38;5;154m_[0m
[5C  
[6A[7C               ___ 
[7C              / ^_)
[7C     _.----._/ /   
[7C    /         /    
[7C __/ (  | (  |     
[7C/__.-|_|--|_|      
[6A 
 [32m1[0m
 [31m1[0m
 [36m0[0m
 
 
[6A[5C   
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m
[5C   
[6A[8C              ___  
[8C             / o_) 
[8C    _.----._/ /    
[8C   /         /     
[8C _/ (  / /  /      
[8C/_/-|_/--|_/       
[6A 
 [32m1[0m
 [31m1[0m
 [36m1[0m
 
 
[6A[5C    
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m
[5C    
[6A[9C              ___  
[9C             / -_) 
[9C     _.----./ /    
[9C    /        /     
[9C __/ (  | (  |     
[9C/__.-'|_|--|_|     
[6A 
 [32m2[0m
 [31m1[0m
 [36m1[0m
 
 
[6A[5C     
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m
[5C     
[6A[10C               ___ 
[10C              / ^_)
[10C     _.----._/ /   
[10C    /         /    
[10C __/ (  | (  |     
[10C/__.-|_|--|_|      
[6A 
 [32m2[0m
 [31m2[0m
 [36m1[0m
 
 
[6A[5C      
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m
[5C      
[6A[11C              ___  
[11C             / o_) 
[11C    _.----._/ /    
[11C   /         /     
[11C _/ (  / /  /      
[11C/_/-|_/--|_/       
[6A 
 [32m2[0m
 [31m2[0m
 [36m2[0m
 
 
[6A[5C       
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m[38;5;214m-[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m[38;5;214m-[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m[38;5;214m-[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m[38;5;214m-[0m
[5C       
[6A[12C              ___  
[12C             / -_) 
[12C     _.----./ /    
[12C    /        /     
[12C __/ (  | (  |     
[12C/__.-'|_|--|_|     
[6A 
 [32m3[0m
 [31m2[0m
 [36m2[0m
 
 
[6A[5C        
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m[38;5;214m-[0m[38;5;208m_[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m[38;5;214m-[0m[38;5;208m_[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m[38;5;214m-[0m[38;5;208m_[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m[38;5;214m-[0m[38;5;208m_[0m
[5C        
[6A[13C               ___ 
[13C              / ^_)
[13C     _.----._/ /   
[13C    /         /    
[13C __/ (  | (  |     
[13C/__.-|_|--|_|      
[6A 
 [32m3[0m
 [31m3[0m
 [36m2[0m
 
 
[6A[5C         
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m[38;5;214m-[0m[38;5;208m_[0m[38;5;208m-[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m[38;5;214m-[0m[38;5;208m_[0m[38;5;208m-[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m[38;5;214m-[0m[38;5;208m_[0m[38;5;208m-[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m[38;5;214m-[0m[38;5;208m_[0m[38;5;208m-[0m
[5C         
[6A[14C              ___  
[14C             / o_) 
[14C    _.----._/ /    
[14C   /         /     
[14C _/ (  / /  /      
[14C/_/-|_/--|_/       
[6A 
 [32m3[0m
 [31m3[0m
 [36m3[0m
 
 
[6A[5C          
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m[38;5;214m-[0m[38;5;208m_[0m[38;5;208m-[0m[38;5;203m_[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m[38;5;214m-[0m[38;5;208m_[0m[38;5;208m-[0m[38;5;203m_[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m[38;5;214m-[0m[38;5;208m_[0m[38;5;208m-[0m[38;5;203m_[0m
[5C[38;5;154m-[0m[38;5;154m_[0m[38;5;148m-[0m[38;5;184m_[0m[38;5;184m-[0m[38;5;214m_[0m[38;5;214m-[0m[38;5;208m_[0m[38;5;208m-[0m[38;5;203m_[0m
[5C          
[6A[15C              ___  
[15C             / -_) 
[15C     _.----./ /    
[15C    /        /     
[15C __/ (  | (  |     
[15C/__.-'|_|--|_|     
[6A 
 
 
 
 
 
